function [w, obj_list] = AsynSparseRidgeRegression(prob, nThread, w0, lbound, ubound, maxEpoch, minObj)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @author:  Yijun Huang                 %
% @email:   yijun.huang.1983@gmail.com  %
% @version: 1.1 (Nov. 2016)             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% objective function:
%  min \|X w - y\|^2 / (2 nSample) + l2 \|w\|_2^2 
%  s.t. lbound <= w <= ubound
%       X is sparse

%% inputs:
%  problem -- the data generated by prob_SparseRidgeRegression()
%  nThread -- The number of threads (positive integer).
%  w0 -- the initial value of varialbes (vector [nFeature,1] or empty [] or double).
%  lbound, ubount -- The lower and upper bound of variables (vector [nFeature,1] or empty [] or double).
%  minObj -- Stopping criteria: the limit of objective function value (double).
%  maxEpoch -- Stopping criteria: the limit the of epochs (positive integer).
%% outputs:
%  w -- The result of variable by trained.
%  obj_list -- The objective function value at each epoch.


%%

inputErr = false;

if (nThread<1)
    inputErr = true;
else
    nThread = uint64(nThread);
end

if (maxEpoch<1)
    inputErr = true;
else
    maxEpoch = uint64(maxEpoch);
end

minObj = double(minObj);

[nSample,nFeature] = size(prob.A);

if ~isempty(w0)
   if isvector(w0)
       if length(w0) ==1
           w0 = w0*ones(nFeature,1);
       else if length(w0) ~= nFeature
               inputErr = true;
           end
       end
   else
       inputErr = true;
   end
end

if ~isempty(lbound)
   if isvector(lbound)
       if length(lbound) ==1
           lbound = lbound*ones(nFeature,1);
       else if length(lbound) ~= nFeature
               inputErr = true;
           end
       end
   else
       inputErr = true;
   end
end

if ~isempty(ubound)
   if isvector(ubound)
       if length(ubound) ==1
           ubound = ubound*ones(nFeature,1);
       else if length(ubound) ~= nFeature
               inputErr = true;
           end
       end
   else
       inputErr = true;
   end
end

if ~(isempty(lbound) || isempty(ubound))
    if sum(lbound>ubound)
        inputErr = true;
    end
end

if inputErr
    fprintf('\nAsynSparseRidgeRegression: error in inputs\n');
    return;
end


%% compile 'sp_aspcd' 

L1 = false;
current = pwd;
cd('../tools')
compile_sp_aspcd(L1, current)
cd(current)

%%
l1 = 0;

[w, obj_list] = m_sp_aspcd(double(nThread), prob.A, prob.A', prob.b, prob.l2, 0, w0, lbound, ubound, double(maxEpoch), minObj); 


return
